from abaqus import*
from abaqusConstants import*
import part, material, section, assembly, step, interaction
import regionToolset, displayGroupMdbToolset as dgm, mesh, load, job

##===========================================================
## INPUT

##===  IMPORTANT  ===###
# The part with initial crack need to have a set of the whole crack part named "crackedPart" 

	## MODEL INPUT
analysisIdentity= 'myIdentity'	# Analysis indentity
caeFile= 'myCae'				# Name of the .cae file
modelName = 'myModel'			# Name of model
partName = 'myPart'				# Name of part (in assembly) containing crack
crackStep='MyCrackStep'			# Name of Step with crack initiation

NameOfMaterial='myMaterial'		# Name of material the material of the part containing crack


	## CRACK INPUT 
crackSize = 1.0					# Initial crack size - IMPORTANT: the mesh size around the crack cannot exceed 3% of crack size
radiusOfCrackTipNode = 0.01		# Radius of the area containg the crack tip - IMPORTANT: Radius of the crack tip node cannot be smaller than crack size

intitialGlobalCrackCoordinates = (0.,0.,0.) # Initial global coordinates of crack initiation

	## CALCULATION INPUT

contours=20						# Number of contours for calculation of SIF
contourAverage = 5     			# Number of contours to include for calculation of the average of contours - IMPORTANT: has to be a positive integer


increment=20					# Number of increments of the crack growth

incrementSize_temp =0.05  		# Size of increments - extension of the crack

# END INPUT

##==========================================================
# Create tuples
crackPropagationDirection =[] 
K1=[]
K2=[]
outputRes = open('XFEMCrackResults'+"_"+modelName+"_"+analysisIdentity+'.txt','a') # Initiate report file
firstLine=['nIncrement'," ",'crackPropagationDirection',"  ",'cracktipX',"  ",'cracktipY',"    ",'K1',"    ",'K2',"   ","\n"] 
outputRes.writelines(firstLine)
outputRes.close()

mdb = openMdb(caeFile+'.cae')
myModel = mdb.models[modelName]
myAssembly = myModel.rootAssembly
myPartInstance = myAssembly.instances[partName]

# XFEM Material properties

myMaterial=myModel.materials[NameOfMaterial]
myMaterial.MaxpsDamageInitiation(table=((1e+30, ), ))
myMaterial.maxpsDamageInitiation.DamageEvolution(table=((1.0, ), ), type=DISPLACEMENT)

#================================================================================

crackLocation=[(0, 0.),(crackSize/2, 0.),(crackSize, 0.)] # Initial crack coordinate(spline points)
localCrack_temp = crackLocation

nIncrement= 0 			#temp start

for nIncrement in range(increment):

	localCrack = tuple(localCrack_temp)

	### Create initial crack ###


	mySketch2 = myModel.ConstrainedSketch(name='CrackSketch',sheetSize=200.0)
	mySketch2.sketchOptions.setValues(viewStyle=REGULAR)
	mySketch2.setPrimaryObject(option=STANDALONE)
	mySketch2.Spline(localCrack)

	myCrack = myModel.Part(name='CrackPart', 
		dimensionality=TWO_D_PLANAR, type=DEFORMABLE_BODY)
	myCrack.BaseWire(sketch=mySketch2)
	mySketch2.unsetPrimaryObject()
	del myModel.sketches['CrackSketch']

	#==========================================================================

	## Create assembly
	myAssembly.Instance(name='myCrack-1', part=myCrack, dependent=OFF)
	myCrackInstance = myAssembly.instances['myCrack-1']
	myCrackInstance.translate(intitialGlobalCrackCoordinates)


	# Create a set for the crack
	edges = myCrackInstance.edges
	myAssembly.Set(edges=edges, name='crackFace')


	#=============================================================================
	## CREATE SETS

	# # Determine the closest node to crack tip

	crackTipCoordinates= ((localCrack[len(localCrack)-1][0]+intitialGlobalCrackCoordinates[0]), 
		(localCrack[len(localCrack)-1][1]+intitialGlobalCrackCoordinates[1]), .0)


	crackTipNodes = myPartInstance.nodes.getByBoundingSphere((crackTipCoordinates), radiusOfCrackTipNode)

	closestindex = 0 # Assume the first index in the crackTipNodes is closest node
	closestX = crackTipNodes[closestindex].coordinates[0]-crackTipCoordinates[0]
	closestY = crackTipNodes[closestindex].coordinates[1]-crackTipCoordinates[1]
	closestdistance = sqrt(pow(closestX,2)+pow(closestY,2))

	for index in range(len(crackTipNodes)):
		distanceX = crackTipNodes[index].coordinates[0]-crackTipCoordinates[0]
		distanceY = crackTipNodes[index].coordinates[1]-crackTipCoordinates[1]
		distance = sqrt(pow(distanceX,2)+pow(distanceY,2))
		if distance < closestdistance:
			closestdistance = distance
			closestindex = index
			
	# The closest node as crack tip
	crackTipNode = myPartInstance.nodes.sequenceFromLabels((crackTipNodes[closestindex].label,))
	myAssembly.Set(nodes=crackTipNode, name='crackTip')
	crackFront = crackTip = myAssembly.sets['crackTip']

	#====================================================================================

	### Define crack ##

	## XFEM crack ##
	myAssembly.engineeringFeatures.XFEMCrack(
		crackDomain=myAssembly.sets['crackedPart'], 
		crackLocation= myAssembly.sets['crackFace'], 
		name='XFEMcrack')
		
	## Crack tip (CI crack) ##
	myAssembly.engineeringFeatures.ContourIntegral(name='ContourCrack', 
		crackFront=crackFront, crackTip=crackTip, 
		extensionDirectionMethod=Q_VECTORS, qVectors=(((localCrack[len(localCrack)-2][0], localCrack[len(localCrack)-2][1],0.0),
		(localCrack[len(localCrack)-1][0], localCrack[len(localCrack)-1][1], 0.0)), ), midNodePosition=0.5)

	#====================================================================================

	# Request history output for the crack
	myModel.HistoryOutputRequest(name='SIF_history', 
		createStepName=crackStep, contourIntegral='ContourCrack',
		numberOfContours=contours, contourType=K_FACTORS, kFactorDirection=KII0, rebar=EXCLUDE, sectionPoints=DEFAULT)

	#====================================================================================

	# Create and submit job
	myAssembly.regenerate()

	myJob = mdb.Job(name=modelName+'_'+analysisIdentity+'_'+str(nIncrement), model=modelName,
		description='Contour integral analysis')
	mdb.saveAs(pathName=modelName+"_"+analysisIdentity+'_'+str(nIncrement))

	myJob.submit(consistencyChecking=OFF)
	myJob.waitForCompletion()



	#***************************************************************************

	#___________ OUTPUT ______________ OUTPUT _______________ OUTPUT ___________

	#***************************************************************************


	# Read from history output

	import odbAccess
	crackODB=session.openOdb(name=modelName+'_'+analysisIdentity+'_'+str(nIncrement), path=modelName+'_'+analysisIdentity+'_'+str(nIncrement)+'.odb', readOnly=True)
	histRegion=crackODB.steps[crackStep].historyRegions['ElementSet  PIBATCH']

	crackPropagationDirectionTemp = 0

	for i in range(1*contours-contourAverage,1*contours):
		crackPropagationDirectionTemp = crackPropagationDirectionTemp + histRegion.historyOutputs[histRegion.historyOutputs.keys()[i]].data[0][1]

	crackPropagationDirection.append(crackPropagationDirectionTemp/(contourAverage))


	## Calculate average K1 ##
	K1_temp = 0
	for i in range(3*contours-contourAverage,3*contours):
		K1_temp = K1_temp + histRegion.historyOutputs[histRegion.historyOutputs.keys()[i]].data[0][1]

	K1.append(K1_temp/(2*contourAverage))  #Divided by 2 as Abaqus over estimate by a factor 2

	## Calculate average K2 ##
	K2_temp = 0
	for i in range(4*contours-contourAverage,4*contours):
		K2_temp = K2_temp + histRegion.historyOutputs[histRegion.historyOutputs.keys()[i]].data[0][1]

	K2.append(K2_temp/(2*contourAverage))   #Divided by 2 as Abaqus over estimate by a factor 2
	
	cracktipX= localCrack_temp[len(localCrack_temp)-1][0]+intitialGlobalCrackCoordinates[0]
	cracktipY= localCrack_temp[len(localCrack_temp)-1][1]+intitialGlobalCrackCoordinates[1]
	
	crackODB.close()
	#====================================================================================
	
	# Calculate the next crack tip from crackPropagationDirection
	q1 = [localCrack[len(localCrack)-1][0]-localCrack[len(localCrack)-2][0], localCrack[len(localCrack)-1][1]-localCrack[len(localCrack)-2][1], 0.0]
	q1len = sqrt(pow(q1[0],2)+pow(q1[1],2))
	q1[0] = q1[0]/q1len
	q1[1] = q1[1]/q1len
	q2 = [0.,0.,0.]
	q2[0] = q1[0]*cos(radians(crackPropagationDirection[nIncrement]))-q1[1]*sin(radians(crackPropagationDirection[nIncrement]))
	q2[1] = q1[0]*sin(radians(crackPropagationDirection[nIncrement]))+q1[1]*cos(radians(crackPropagationDirection[nIncrement]))
	
	#===================================================================================
	# Save the results 
	# nIncrement crackPropagationDirection cracktipX cracktipY K1 K2 

	outputRes = open('XFEMCrackResults'+"_"+modelName+"_"+analysisIdentity+'.txt','a')
	lines = [str(nIncrement),"   ",
		str(crackPropagationDirection[len(crackPropagationDirection)-1]),"   ",
		str(cracktipX),"   ",
		str(cracktipY),"   ",
		str(K1[len(K1)-1]),"   ",
		str(K2[len(K2)-1]),"   ",
		"\n"]
	outputRes.writelines(lines)
	outputRes.close()

	localCrack_temp.append((localCrack_temp[len(localCrack_temp)-1][0]+incrementSize_temp*q2[0],localCrack_temp[len(localCrack_temp)-1][1]+incrementSize_temp*q2[1]))

print ('Simulation completed!')